import unittest
from flask_testing import TestCase
from datetime import datetime

from website.helpfunctions.database import *
from website.helpfunctions.sorting_and_filtering import *
from website.helpfunctions.statistics import *
from website.helpfunctions.dymmyData import *

from website.models import Question, User, Answer, Keyword
from website.stateEnum import QuestionEnum
from website import create_app, db
from flask import url_for, request


app = create_app()

class TestHelpFunctions(TestCase):
    def create_app(self):
        app.config['TESTING'] = True
        app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///test.db'
        app.config['SECRET_KEY'] = 'abc'
        return app

    def setUp(self):  #Lage ein test database TODO
        db.create_all()
        user = User(email='testuser@example.com', password='testpassword')
        
        db.session.add(user)
        db.session.add(Answer(answer="", lable="find_prices2", description="", state=AnswerEnum.STANDARD.value))
        db.session.add(Answer(answer="", lable="print_tickets2", description="", state=AnswerEnum.STANDARD.value))
        db.session.add(Answer(answer="", lable="buy_tickets2", description="", state=AnswerEnum.SEASON.value))
        db.session.add(Answer(answer="", lable="buy_tickets3", description="", state=AnswerEnum.SEASON.value))

        db.session.add(Keyword(word="Hello",answer_id=3))
        db.session.add(Keyword(word="x",answer_id=3))
        db.session.add(Keyword(word="19",answer_id=4))
        db.session.add(Keyword(word="Covid",answer_id=4))

        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        db.session.add(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        db.session.add(Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=2, confidence=57.3,state=QuestionEnum.TRAINED_ON.value))
        db.session.add(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=82.3,state=QuestionEnum.TRAINED_ON.value))
        db.session.add(Question(timestamp=time + relativedelta(months=2), message="", bot_answer_id=1, true_answer_id=2, confidence=100.0,state=QuestionEnum.NOT_VALIDATED.value))
        db.session.add(Question(timestamp=time + relativedelta(months=2), message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.NOT_VALIDATED.value))
        db.session.add(Question(timestamp=time + relativedelta(months=3), message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.NOT_VALIDATED.value))
        db.session.add(Question(timestamp=time + relativedelta(months=4), message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.NOT_VALIDATED.value))

        db.session.commit()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    #login
    def test_login(self):
        response = self.client.post(url_for('auth.login'), data={'email': 'testuser@example.com', 'password': 'testpassword'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(request.url_root[:-1] + url_for('views.home'), request.url)
    
    def test_index(self):
        with app.test_client() as client:
            response = client.get('/')
            self.assertEqual(response.status_code, 302)

    # Sorting and filtring functions ----------------------------------------------------------------------------------------------
    def test_is_validated(self):
        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        self.assertTrue(is_validated(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value)))
        self.assertTrue(not is_validated(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.NOT_VALIDATED.value)))
        self.assertTrue(not is_validated(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.TRAINED_ON.value)))

    def test_is_trainedOn(self):
        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        self.assertTrue(not is_trained_on(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value)))
        self.assertTrue(not is_trained_on(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.NOT_VALIDATED.value)))
        self.assertTrue(is_trained_on(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.TRAINED_ON.value)))

    def test_is_active(self):
        self.assertTrue(is_active(Answer(answer="", lable="", description="", state=AnswerEnum.STANDARD.value)))
        self.assertTrue(is_active(Answer(answer="", lable="", description="", state=AnswerEnum.SEASON.value)))
        self.assertTrue(not is_active(Answer(answer="", lable="", description="", state=AnswerEnum.INACTIVE_STANDARD.value)))
        self.assertTrue(not is_active(Answer(answer="", lable="", description="", state=AnswerEnum.INACTIVE_SEASON.value)))

    def test_sort_questions_by(self):
        questions = [
                Question(timestamp=random_datetime(), message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value),
                Question(timestamp=random_datetime(), message="", bot_answer_id=2, true_answer_id=2, confidence=57.3,state=QuestionEnum.VALIDATED.value),
                Question(timestamp=random_datetime(), message="", bot_answer_id=1, true_answer_id=2, confidence=82.3,state=QuestionEnum.VALIDATED.value),
                Question(timestamp=random_datetime(), message="", bot_answer_id=1, true_answer_id=2, confidence=100.0,state=QuestionEnum.VALIDATED.value),
                Question(timestamp=random_datetime(), message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.VALIDATED.value),
            ]
        # Timestamp
        questionsSortByTime = sort_questions_by(questions, "timestamp")
        for i in range(len(questionsSortByTime)-1):
            self.assertTrue(questionsSortByTime[i].timestamp <= questionsSortByTime[i+1].timestamp)
        # bot_answer
        questionsSortByTime = sort_questions_by(questions, "bot_answer")
        for i in range(len(questionsSortByTime)-1):
            self.assertTrue(questionsSortByTime[i].bot_answer_id <= questionsSortByTime[i+1].bot_answer_id) 
        # Innvalide
        self.assertEqual(sort_questions_by(questions, "XXXXXXXXX"), [])

    def test_filterQuestionsWithTimestamp(self):
        startTime = datetime(year=2020,month=4,day=15,hour=20,minute=20)
        endTime = datetime(year=2021,month=1,day=30,hour=10,minute=20)

        questions = [
            Question(timestamp=datetime(year=2020,month=7,day=30,hour=10,minute=20), message="",
                      bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value)
        ]
        self.assertEqual(len(filter_questions_with_timestamp(questions, startTime, endTime)), 1)
        #On startTime        
        questions.append(Question(timestamp=datetime(year=2020,month=4,day=15,hour=20,minute=20), message="",
                      bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(len(filter_questions_with_timestamp(questions, startTime, endTime)), 2)
        #On endTime
        questions.append(Question(timestamp=datetime(year=2021,month=1,day=30,hour=10,minute=20), message="",
                      bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(len(filter_questions_with_timestamp(questions, startTime, endTime)), 3)
        #After endTime
        questions.append(Question(timestamp=datetime(year=2021,month=2,day=24,hour=10,minute=20), message="",
                      bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(len(filter_questions_with_timestamp(questions, startTime, endTime)), 3)
        #Before startTime
        questions.append(Question(timestamp=datetime(year=2020,month=3,day=20,hour=10,minute=20), message="",
                      bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(len(filter_questions_with_timestamp(questions, startTime, endTime)), 3)

    def test_filterQuestionsWithBotAnswer(self):
        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        questions = [
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=2, confidence=57.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=82.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=100.0,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.VALIDATED.value)
        ]
        self.assertEqual(len(filter_questions_with_bot_answer(questions, 'all')), 5) 
        self.assertEqual(len(filter_questions_with_bot_answer(questions, get_answer(1).lable)), 3)
        self.assertEqual(len(filter_questions_with_bot_answer(questions, get_answer(2).lable)), 2)

    def test_filterByAnswerLable(self):
        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        #answer1 = Answer(id= 1, answer="", lable="", description="")    #id 1

        questions = [
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=2, confidence=57.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=82.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=100.0,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=3, true_answer_id=3, confidence=23.3,state=QuestionEnum.VALIDATED.value)
        ]
        self.assertEqual(len(filter_by_answer_lable(questions, 'all')), 6) 
        self.assertEqual(len(filter_by_answer_lable(questions, get_answer(1).lable)), 2)
        self.assertEqual(len(filter_by_answer_lable(questions, get_answer(2).lable)), 3)

    def test_is_season(self):
        self.assertEqual(is_season("Covid"), 4)
        self.assertEqual(is_season("19"), 4)
        self.assertEqual(is_season("x"), 3) 
        self.assertEqual(is_season("Hello"), 3)
        self.assertEqual(is_season("volvo"), None) 
        self.assertEqual(is_season("He"), None) 
        self.assertEqual(is_season("198"), 4) 
        self.assertEqual(is_season("xy"), 3)
        self.assertEqual(is_season("HELLO"), 3)
        self.assertEqual(is_season("hello"), 3)

    def test_is_safe(self):
        self.assertTrue(is_safe("Hei alt vell med deg?"))
        self.assertTrue(is_safe(" <a href=\"https://www.w3schools.com\">Visit W3Schools.com!</a> "))
        self.assertTrue(is_safe("Hei alt vell med deg?"))
        self.assertFalse(is_safe("<script> document.getElementById(\"demo\").innerHTML = \"Hello JavaScript!\"; </script> "))
        self.assertFalse(is_safe("<Script> document.getElementById(\"demo\").innerHTML = \"Hello JavaScript!\"; </Script> "))
        self.assertTrue(is_safe("<p class=\"submit\"><input type=\"submit\" name=\"commit\" value=\"Search\"></p>"))

    def test_to_lable(self):
        self.assertEqual(to_lable("@3beb"), "3beb")
        self.assertEqual(to_lable("__eWQ!!-:,"), "eWQ")
        self.assertEqual(to_lable("e  e"), "ee")
        self.assertEqual(to_lable("\"\\/#¤h%&/(e)=?+i`^*_D:;u,."), "heiDu")
        self.assertEqual(to_lable(" "), "")

    def test_correct_label(self):
        self.assertEqual(correct_label("festspillgavekort"), "gavekort")
        self.assertEqual(correct_label("unknown"), "unknown")
        self.assertEqual(correct_label("ticketing_request"), "buy_tickets")
        self.assertEqual(correct_label("true "), "true")
        self.assertEqual(correct_label("FiB Fordel-info "), "find_prices")

    def test_toggle_answer_state(self):
        self.assertEqual(get_answer(1).state, AnswerEnum.STANDARD.value)
        toggle_answer_state(get_answer(1))
        self.assertEqual(get_answer(1).state, AnswerEnum.INACTIVE_STANDARD.value)
        toggle_answer_state(get_answer(1))
        self.assertEqual(get_answer(1).state, AnswerEnum.STANDARD.value)


        self.assertEqual(get_answer(3).state, AnswerEnum.SEASON.value)
        toggle_answer_state(get_answer(3))
        self.assertEqual(get_answer(3).state, AnswerEnum.INACTIVE_SEASON.value)
        toggle_answer_state(get_answer(3))
        self.assertEqual(get_answer(3).state, AnswerEnum.SEASON.value)

# Statistics ------------------------------------------------------------------------------------------------------------
    def test_getAccuracy(self):
        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        questions = [
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=2, confidence=57.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=82.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=100.0,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=1, confidence=23.3,state=QuestionEnum.VALIDATED.value),
        ]
        self.assertEqual(get_accuracy(questions), 0.4)
        questions.append(Question(timestamp=time, message="", bot_answer_id=3, true_answer_id=3, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(get_accuracy(questions), 0.5)
        questions.append(Question(timestamp=time, message="", bot_answer_id=3, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(round(get_accuracy(questions), 2), 0.43)
        questions.append(Question(timestamp=time, message="", bot_answer_id=3, true_answer_id=3, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(get_accuracy(questions), 0.5)
        self.assertEqual(get_accuracy([]), 0)
        self.assertEqual(get_accuracy([
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=23.3,state=QuestionEnum.VALIDATED.value)
            ]), 1)
        self.assertEqual(get_accuracy([
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=23.3,state=QuestionEnum.VALIDATED.value),
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=3, confidence=23.3,state=QuestionEnum.NOT_VALIDATED.value),
            ]), 1)

    def test_ConfusionMatrixToDim(self):
        time = datetime(year=2020,month=2,day=2,hour=20,minute=20)
        answer1 = get_answer(1)#Answer(id= 1, answer="", lable="", description="")    #id 1
        questions = [
            Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value)
        ]
        self.assertEqual(confusion_matrix_to_dim(questions, answer1), [[1,0],[0,0]])

        questions.append(Question(timestamp=time, message="", bot_answer_id=2, true_answer_id=3, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(confusion_matrix_to_dim(questions, answer1), [[1,0],[0,1]])

        questions.append(Question(timestamp=time, message="", bot_answer_id=3, true_answer_id=3, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(confusion_matrix_to_dim(questions, answer1), [[1,0],[0,2]])

        questions.append(Question(timestamp=time, message="", bot_answer_id=1, true_answer_id=2, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(confusion_matrix_to_dim(questions, answer1), [[1,0],[1,2]])

        questions.append(Question(timestamp=time, message="", bot_answer_id=3, true_answer_id=1, confidence=74.3,state=QuestionEnum.VALIDATED.value))
        self.assertEqual(confusion_matrix_to_dim(questions, answer1), [[1,1],[1,2]])        

    def test_Recall(self):
        #truePositive / (truePositive + falseNegative)
        self.assertEqual(recall([[4,6],[100,4]]), 0.4) #4/(4+6)
        self.assertEqual(recall([[4,0],[3,431]]), 1)
        self.assertEqual(recall([[4,6],[3,431]]), 0.4)
        self.assertEqual(recall([[0,0],[0,0]]), 0)
        self.assertEqual(recall([[0,5],[3,431]]), 0)
        self.assertEqual(recall([[5,5],[3,431]]), 0.5) #5/(5+5)
        self.assertEqual(recall([[-5,-5],[3,431]]), 0)
        self.assertEqual(recall([[-5,5],[3,431]]), 0)
        self.assertEqual(recall([[5,-5],[3,431]]), 0)
    
    def test_precision(self):
        #truePositive / (truePositive + falsePositive)
        self.assertEqual(precision([[4,6],[96,4]]), 0.04) #4/(4+96)
        self.assertEqual(precision([[4,6],[0,4]]), 1) 
        self.assertEqual(precision([[0,0],[0,0]]), 0)
        self.assertEqual(precision([[-5,5],[-3,431]]), 0)
        self.assertEqual(precision([[-5,5],[3,431]]), 0)
        self.assertEqual(precision([[5,5],[-3,431]]), 0)
        self.assertEqual(precision([[5,2],[5,431]]), 0.5)

    def test_get_f1_score(self):
        #2 * (Precision * Recall) / (Precision + Recall)
        self.assertEqual(get_f1_score([[0,0],[0,0]]), 0)
        self.assertEqual(get_f1_score([[4,0],[0,0]]), 1)
        self.assertEqual(round(get_f1_score([[4,4],[0,0]]),2), 0.67)
        self.assertEqual(round(get_f1_score([[4,0],[4,0]]),2), 0.67)
        self.assertEqual(get_f1_score([[0,4],[4,0]]), 0)

    def test_get_num_of_questions(self):
        self.assertEqual(get_num_of_questions(QuestionEnum.TRAINED_ON.name), 2)
        self.assertEqual(get_num_of_questions(QuestionEnum.NOT_VALIDATED.name), 4)
        self.assertEqual(get_num_of_questions(QuestionEnum.VALIDATED.name), 1)

    def test_get_the_increase_of_question(self):
        currentDate = datetime(year=2020,month=6,day=1,hour=20,minute=20)

        actual_counts, actual_dates = get_the_increase_of_question(1, currentDate)
        self.assertEqual(actual_counts, [3, 5, 6, 7])
        self.assertEqual(actual_dates, ["Mar 2020", "Apr 2020", "May 2020", "Jun 2020"])

        actual_counts, actual_dates = get_the_increase_of_question(2, currentDate)
        self.assertEqual(actual_counts, [5, 7])
        self.assertEqual(actual_dates, ["Apr 2020", "Jun 2020"])

        actual_counts, actual_dates = get_the_increase_of_question(4, currentDate)
        self.assertEqual(actual_counts, [7])
        self.assertEqual(actual_dates, ["Jun 2020"])

    def test_get_number_of_years(self):
        self.assertEqual(get_number_of_years(datetime(year=2020,month=12,day=30,hour=20,minute=20)), 1)
        self.assertEqual(get_number_of_years(datetime(year=2020,month=2,day=2,hour=20,minute=20)), 1)
        self.assertEqual(get_number_of_years(datetime(year=2021,month=2,day=1,hour=20,minute=20)), 1)
        self.assertEqual(get_number_of_years(datetime(year=2021,month=2,day=3,hour=20,minute=20)), 2)
        self.assertEqual(get_number_of_years(datetime(year=2022,month=4,day=2,hour=20,minute=20)), 3)
        self.assertEqual(get_number_of_years(datetime(year=2020,month=1,day=1,hour=20,minute=20)), 0)
        self.assertEqual(get_number_of_years(datetime(year=2001,month=1,day=1,hour=20,minute=20)), 0)

    def test_generate_colors(self):
        self.assertEqual(len(generate_colors(5)), 5)
        self.assertEqual(len(generate_colors(1000)), 1000)
        self.assertEqual(len(generate_colors(-5)), 0)




        




    
    