from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from os import path,makedirs,chmod, stat
from flask_login import LoginManager
from logging.config import dictConfig

#Sesion
from flask import Flask
from flask_session import Session #pip install Flask-Session

db = SQLAlchemy()
DB_NAME = "database.db"

def create_app():
    app = Flask(__name__)
    app.config['SECRET_KEY'] = 'abc'
    app.config['SQLALCHEMY_DATABASE_URI'] = f'sqlite:///{DB_NAME}'
    app.config['APPLICATION_LOG_PATH'] = 'logs/'
    db.init_app(app)

    #Sesion
    app.config["SESSION_PERMANENT"] = False
    app.config["SESSION_TYPE"] = "filesystem"
    Session(app)

    from .views import views
    from .auth import auth
    #from fib_chatbot.src.app2 import app2

    app.register_blueprint(views,url_prefix='/')
    app.register_blueprint(auth,url_prefix='/')
    #app.register_blueprint(app2,url_prefix='/fib_chatbot/')

    from .models import User
    

    create_database(app)
    create_logs(app)

    login_manager = LoginManager()
    login_manager.login_view = 'auth.login'
    login_manager.init_app(app)

    @login_manager.user_loader
    def load_user(id):
        return User.query.get(int(id))
    
    return app

def create_logs(app):
    # Create the log directory if it does not exist
    if not path.exists(app.config['APPLICATION_LOG_PATH']):
        makedirs(app.config['APPLICATION_LOG_PATH'])
        chmod(app.config['APPLICATION_LOG_PATH'], 0o777)

    # Check if the log files exist and have the required permissions
    log_files = [
        app.config['APPLICATION_LOG_PATH'] + '/info.log',
        app.config['APPLICATION_LOG_PATH'] + '/error.log'
    ]
    for log_file in log_files:
        if not path.exists(log_file):
            open(log_file, 'w').close()
            chmod(log_file, 0o666)
        else:
            file_stat = stat(log_file)
            if not file_stat.st_mode & 0o777 == 0o666:
                chmod(log_file, 0o666)

    # Configure the logger
    logging_configuration = {
        'version': 1,
        'handlers': {
            'info_file_handler': {
                'class': 'logging.handlers.RotatingFileHandler',
                'level': 'INFO',
                'formatter': 'standard',
                'filename': app.config['APPLICATION_LOG_PATH'] + '/info.log',
                'maxBytes': 1024 * 1024 * 100,  # 100 MB
                'backupCount': 20,
                'encoding': 'utf8'
            },
            'error_file_handler': {
                'class': 'logging.handlers.RotatingFileHandler',
                'level': 'ERROR',
                'formatter': 'standard',
                'filename': app.config['APPLICATION_LOG_PATH'] + '/error.log',
                'maxBytes': 1024 * 1024 * 100,  # 100 MB
                'backupCount': 20,
                'encoding': 'utf8'
            },
            'console': {
                'class': 'logging.StreamHandler',
                'level': 'INFO',
                'formatter': 'standard',
                'stream': 'ext://sys.stdout'
            }
        },
        'loggers': {
            'FiB-CA_log': {
                'level': 'INFO',
                'handlers': ['info_file_handler', 'error_file_handler', 'console']
            }
        },
        'formatters': {
            'standard': {
                'format': '[%(asctime)s] %(levelname)s in %(module)s: %(message)s'
            }
        }
    }

    dictConfig(logging_configuration)

def create_database(app):
    if not path.exists('website/' + DB_NAME):
        with app.app_context():
            db.create_all()
        print('Created database.')
