from flask import Blueprint, render_template, request, flash, redirect, url_for
from flask_login import login_user, logout_user, current_user
from werkzeug.security import generate_password_hash, check_password_hash
from .helpfunctions.authorization import check_for_inactivity_logout, custom_login_required
from .models import User
from . import db

auth = Blueprint('auth',__name__)
@auth.route('/login',methods=['GET','POST'])
def login():
    if request.method == 'POST':
        email = request.form.get('email')
        password = request.form.get('password')
        user = User.query.filter_by(email=email).first()
        if user:
            if check_password_hash(user.password, password):
                flash('Innlogging vellykket', category='success')
                login_user(user,remember=True)
                return redirect(url_for('views.home'))
            else:
                flash('Feil passord. Prøv igjen',category='error')
        else:
            flash('Denne emailen finnes ikke.', category='error')
    return render_template("login.html", user = current_user)

@auth.route('/sign_up',methods=['GET','POST'])
def sign_up():
    if request.method == 'POST':
        first_name = request.form.get('first_name')
        last_name = request.form.get('last_name')
        email = request.form.get('email')
        password1 = request.form.get('password1')
        password2 = request.form.get('password2')
        user = User.query.filter_by(email=email).first()
        if user:
            flash('Email eksisterer allerede.', category='error')
        elif password1 != password2:
            flash('Passord matcher ikke.', category='error')
        else:
            new_user = User(first_name=first_name,last_name=last_name,email=email,password=generate_password_hash(password1,method='sha256'))
            db.session.add(new_user)
            db.session.commit()
            flash('Ny bruker opprettet', category='success')
            login_user(new_user,remember=True)
            return redirect(url_for('views.home'))
    return render_template('sign_up.html', user=current_user)


@auth.route('/logout',methods=['GET','POST'])
@custom_login_required(show_flash=False)
def logout():
    auto_logout = request.args.get('auto', default=0, type=int)
    logout_user()

    if auto_logout:
        check_for_inactivity_logout() 
        flash('Du har blitt logget ut pga. inaktivitet.', category='info-inactivity')
    else:
        flash('Du har blitt logget ut.', category='success')

    return redirect(url_for('auth.login'))