from flask_login import current_user, logout_user
from datetime import datetime
from functools import wraps
from flask import flash, redirect, url_for

def custom_login_required(show_flash = True):
    """check whether the user is logged in, and gives an error message if he is not logged in"""
    def inner_decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            if not current_user.is_authenticated:
                if show_flash:
                    flash('Logg inn for å vise denne siden.', 'error')
                return redirect(url_for('auth.login'))
            return f(*args, **kwargs)
        return decorated_function
    return inner_decorator


def check_for_inactivity_logout():
    """If user has been inactiv for more than 30 seconds the user is logget out"""
    if current_user.is_authenticated:
        last_active = current_user.get_last_active()
        if last_active is not None and (datetime.datetime.now() - last_active).seconds > 30:
            logout_user()
            flash('Du har blitt logget ut pga. inaktivitet.', 'info-inactivity')
            return redirect(url_for('auth.login'))