from ..models import Question, Answer
from ..stateEnum import QuestionEnum, AnswerEnum
from datetime import datetime
from .sorting_and_filtering import correct_label
from .database import get_question, get_answer_by_label
from random import randint
from typing import List
from .. import db
import json
import csv
import os


def random_answer() -> Answer:
    """ Function returns a random answer from the database. If the database is empty it returns Answer() """
    answer = Answer.query.all()
    if(len(answer) != 0):
        num = randint(0,len(answer)-1)
        return answer[num]
    else:
        print('Error in randomAnswer: No answer in database')
        return Answer()

def random_datetime() -> datetime:
    """Give a random date in 2022"""
    randomYear = randint(2022,2023)
    randomMonth = randint(1,12)
    randomDay = randint(1,28)

    randomHour = randint(0,23)
    randomMinute = randint(0,59)
    return datetime(year=randomYear,month=randomMonth,day=randomDay,hour=randomHour,minute=randomMinute)

def gennerate_answer():
    """Function fore generating all answers"""
    answers = Answer.query.all()
    if len(answers) < 3:
        #Find prices
        db.session.add(Answer(
            answer='Ordinære priser er oppgit på nettsiden for hvert arrangement. på <a href="https://fib.no">Fib.no</a> finner du også en oversikt over tilbu og rabatter samt vilkår.', 
            lable="find_prices", description="Finn priser", state=AnswerEnum.STANDARD.value))
        #Print tickets
        db.session.add(Answer(
            answer='Det er fint om du benytter din digitale billett på mobilen, men det er også mulig å skrive ut e-billetten på forhånd. Hvis du bruker mobilen, er det fint om du sørger for at lysstyrken er skrudd opp på maks.', 
            lable="print_tickets", description="Fysisk/digital bilett", state=AnswerEnum.STANDARD.value))
        #Buy tickets
        db.session.add(Answer(
            answer="For å kjøpe billett kan du klikke på Kjøp billett på nettsiden for hvert <a href='https://www.fib.no/program/'>arrangement</a>. Her kan man også se om det er få ledige plasser eller utsolgt. I festivalperioden (25. mai-07.juni) kan du også kjøpe billetter i vår billettbod på Torgallmenningen 10:00-20:00.", 
            lable="buy_tickets", description="Kjøp av biletter", state=AnswerEnum.STANDARD.value))
        #Festspillkort
        db.session.add(Answer(
            answer="<a href='https://www.fib.no/billettinfo/billett/#Billettpriser'>FiB Fordel</a> er et medlemskap som gir inntil 30 % rabatt på billetter og en rekke andre fordeler. ", 
            lable="festspillkort", description="Festspillkort", state=AnswerEnum.STANDARD.value))
        #Festival dates
        db.session.add(Answer(
            answer="Festspillene i Bergen er fra 24. mai til 7. juni", 
            lable="find_festival_dates", description="Festival datoer", state=AnswerEnum.STANDARD.value))
        #Gavekort
        db.session.add(Answer(
            answer="Informasjon om hvordan du kjøper og bruker <a href='https://www.fib.no/billettinfo/gavekort/'>Festspillenes digitale gavekort finner du her</a>.", 
            lable="gavekort", description="Gavekort", state=AnswerEnum.STANDARD.value))
        #Get_bicycle
        db.session.add(Answer(
            answer="Håper jeg har vært til nytte! Hvis ikke, er du velkommen til å kontakte oss her. Ha en fine dag!", 
            lable="get_bicycle", description="Avsluttende Samtale", state=AnswerEnum.STANDARD.value))
        #Dress code
        db.session.add(Answer(
            answer="Tidligere var det mange som kledde seg opp for å gå på konserter og forestillinger. Du kan fortsatt pynte deg hvis du vil, men i dag er det mye vanligere å se publikum uformelt kledd enn i bunad. Kom som du vil, det viktigste er at du føler deg vel!", 
            lable="get_dress_code", description="Kleskode", state=AnswerEnum.STANDARD.value))
        #Refund
        db.session.add(Answer(
            answer="Festspillene refunderer billetter hvis vi må avlyse eller endre dato på arrangementet, ikke ved endring av artist, forsinkelse eller mindre teknisk feil. Du kan ikke få pengene tilbake hvis du ikke har anledning til å gå på et arrangement, men du kan gjerne gi billetten videre til noen andre. Les mer om <a href='https://www.fib.no/billettinfo/billett/#Avlyst'>refunderinger her</a>. ", 
            lable="get_refund", description="Refusjon", state=AnswerEnum.STANDARD.value))
        #Program
        db.session.add(Answer(
            answer="Du finner informasjon om hvert enkelt arrangement på fib.no/program. Her  finner du detaljer som klokkeslett for arrangementsstart, arenaadresse, bevertning, transport og tilgjengelighet. På våre programsider har du mulighet til å sortere arrangementer basert på blant annet dato, sjanger og arena. Meld deg gjerne på vårt nyhetsbrev for å holde deg oppdatert på vårt program!", 
            lable="program_content", description="Program", state=AnswerEnum.STANDARD.value))
        #Volunteer
        db.session.add(Answer(
            answer="Festspillene i Bergen søker hvert år rundt 200 frivillige. De er maskineriet som får festivalen til å fungere og du kan delta i mange ulike roller. Les mer om vår <a href='https://www.fib.no/frivillig/'>frivilligordning her</a>. ", 
            lable="volunteer", description="Frivillig arbeid", state=AnswerEnum.STANDARD.value))
        #Helsing
        db.session.add(Answer(
            answer="dette er svaret", 
            lable="true", description="Helsing", state=AnswerEnum.STANDARD.value))
        #Unknown
        db.session.add(Answer(
            answer="Hmm... Jeg er bare en chatbot og ikke et menneske, og dette klarer eg ikke å svare på. Kanskje finner du svaret du leter etter under ofte stilte spørsmål? (linjeskift) Ved spørsmål om billetter kontakt oss gjerne på tlf. 55 21 06 30, biletten@fib.no eller chat med en ansatt ved å klikke på boksen øverst i dette vinduet (linjeskifte) Finner du ikke svaret ditt og er usikker på hvor du kan henvende deg? Her finer du mer kontaktinfo.", 
            lable="unknown", description="Ukjent", state=AnswerEnum.STANDARD.value))
        db.session.commit() 

def generate_questions():
    """Function for generating 5 questions"""
    answers = Answer.query.all()
    if len(answers) < 3:
        gennerate_answer()

    for i in range(50):
        db.session.add(Question(timestamp=random_datetime(), message="Når er festspillene for 2024?", bot_answer_id=1, 
                                true_answer_id=1, confidence=74.3, state=QuestionEnum.VALIDATED.value))
        db.session.add(Question(timestamp=random_datetime(), message="Når skal Justin Bieber synga?", bot_answer_id=1, 
                                true_answer_id=random_answer().id, confidence=57.3, state=QuestionEnum.VALIDATED.value))
        db.session.add(Question(timestamp=random_datetime(), message="Hvordan kommer en seg til grigg hallen?", 
                                bot_answer_id=random_answer().id, true_answer_id=random_answer().id, confidence=82.3, state=QuestionEnum.VALIDATED.value))
        db.session.add(Question(timestamp=random_datetime(), message="Hvem har laget denne flotte chatbotten?", 
                                bot_answer_id=random_answer().id, true_answer_id=random_answer().id, confidence=100.0, state=QuestionEnum.NOT_VALIDATED.value))
        db.session.add(Question(timestamp=random_datetime(), message="Hvordan fungere denne chatboten her når spørsmålene blir alt for lang for boksen de skal vises frem i? Jeg trur dette ikke blir å funke.", 
                                bot_answer_id=3, true_answer_id=3, confidence=23.3, state=QuestionEnum.NOT_VALIDATED.value))
    
    db.session.commit()

# Used to get questions from datadump/jsonQuery.json
def get_questions_from_json(path) -> List[Question]:
    """ Takes the path to a JSON file containing "Id", "Timestamp", "Message", "Label" and "Confidence", and returns a list of question objects """
    print('Getting questions from JSON')
    with open(path, encoding='utf-8') as f:
        data = json.load(f)

    questions = []

    for entry in data:
        entry_id = entry['Id']
        entry_timestamp = entry['Timestamp']
        entry_message = entry['Message']
        entry_label = entry['Label']
        entry_confidence = entry['Confidence']

        entry_label = correct_label(entry_label)

        id = int(entry_id)
        timestamp = datetime.strptime(entry_timestamp[:-1], '%Y-%m-%dT%H:%M:%S.%f')
        message = entry_message
        answer_id = get_answer_by_label(entry_label).id
        confidence = float(entry_confidence)
        state = QuestionEnum.NOT_VALIDATED.value

        new_question = Question(id=id,timestamp=timestamp,message=message,bot_answer_id=answer_id,true_answer_id=answer_id,confidence=confidence,state=state)
        questions.append(new_question)
    return questions

# Used to get questions from datadump/csvQuery.csv
def get_questions_from_csv(path) -> List[Question]:
    """ Takes the path to a CSV file with the structure Id,Timestamp,Message,Label,Confidence and returns a list of question objects """
    print('Getting questions from CSV')
    questions = []
    with open(path, 'r') as file:
        csvreader = csv.reader(file)
        header = next(csvreader)
        for row in csvreader:
            entry_id = row[0]
            entry_timestamp = row[1]
            entry_message = row[2]
            entry_label = row[3]
            entry_confidence = row[4]

            entry_label = correct_label(entry_label)

            id = int(entry_id)
            timestamp = datetime.strptime(entry_timestamp[:-1], '%Y-%m-%dT%H:%M:%S.%f')
            message = entry_message
            answer_id = get_answer_by_label(entry_label).id
            confidence = float(entry_confidence)
            state = QuestionEnum.NOT_VALIDATED.value

            new_question = Question(id=id,timestamp=timestamp,message=message,bot_answer_id=answer_id,true_answer_id=answer_id,confidence=confidence,state=state)
            questions.append(new_question)
    return questions

def fill_database_with_json():
    """Add questions from json to the database"""
    print('Filling database with questions')
    absolute_path = os.path.dirname(__file__)
    last_dir, remaining_path = os.path.split(absolute_path)
    relative_path = "datadump/jsonQuery.json"
    full_path = os.path.join(last_dir, relative_path)
    questions = get_questions_from_json(full_path)

    for question in questions:

        exsisting_question = get_question(question.id)
        if exsisting_question:
            exsisting_question.timestamp = question.timestamp
            exsisting_question.message = question.message
            exsisting_question.bot_answer_id = question.bot_answer_id
            exsisting_question.true_answer_id = question.true_answer_id
            exsisting_question.confidence = question.confidence
            exsisting_question.state = question.state
            db.session.commit()
        else:
            db.session.add(question)
            db.session.commit()