from ..models import Question
from ..stateEnum import QuestionEnum
from .database import *
from .sorting_and_filtering import correct_label
from typing import List
import csv
import os

from ..retrain.preproses_data import proscess_questions,remove_labels
from ..retrain.train_new_model import produce_modell

# Used to get questions from datadump/questions.csv
def get_train_from_csv(path) -> List[Question]:
    """ Creates a list of Questions that only has a message and true_answer_id value. This is for CSV-files that only store value and answer"""
    print('Getting questions from CSV')
    questions_from_csv = []
    with open(path, 'r') as file:
        csvreader = csv.reader(file)
        header = next(csvreader)
        for row in csvreader:
            entry_message = row[0]
            entry_label = row[1]

            message = entry_message
            answer = correct_label(entry_label) #str, but
            
            # if entry lable is not null
            try:
                answer_id = get_answer_by_label(answer).id
                new_question = Question(message=message,true_answer_id=answer_id,state=QuestionEnum.VALIDATED.value)
                questions_from_csv.append(new_question)
            except:
                print("Error in getTrainFromCsv: Label '"+entry_label+"' is not valid.")

    return questions_from_csv



# Train new model -------------------------------------------------------------------------------------------------------

def train_with_csv_and_db(csv_data_path, proscessed_data_path, model_path):
    """ Takes the path to a CSV file with two values(value,answer) and all questions from the db that are validated or trained on ,and trains a new model using this """
    # Getting the questions from the csv file
    questions_from_csv = get_train_from_csv(csv_data_path)

    # Getting the questions from the database that can be included in the trainingdata.
    questions_from_db = get_questions_by_state(QuestionEnum.VALIDATED.name) + get_questions_by_state(QuestionEnum.TRAINED_ON.name)
    questions = questions_from_csv + questions_from_db

    # Making sure the path for the proscessed data exists
    if not os.path.exists(proscessed_data_path):
        os.makedirs(proscessed_data_path)
    # Making sure the path for the model exists
    if not os.path.exists(model_path):
        os.makedirs(model_path)

    # Trains model on questions
    proscess_questions(questions,proscessed_data_path)
    remove_labels(proscessed_data_path + 'processed_data.csv', minimum_occurrence=5)
    produce_modell(proscessed_data_path + 'processed_data.csv',model_path)

    # Creates db entry for training event
    add_trained_entry(len(questions))

    # Changes the state of the questions from the db to trained_on
    change_state_of_question_to(questions_from_db,QuestionEnum.TRAINED_ON.value)

def train_with_db(proscessed_data_path, model_path):
    """ Trains a new model using all questions from the database that are validated or trained on """
    # Getting the questions from the database that can be included in the trainingdata.
    questions_from_db = get_questions_by_state(QuestionEnum.VALIDATED.name) + get_questions_by_state(QuestionEnum.TRAINED_ON.name)

    # Making sure the path for the proscessed data exists
    if not os.path.exists(proscessed_data_path):
        os.makedirs(proscessed_data_path)
    # Making sure the path for the model exists
    if not os.path.exists(model_path):
        os.makedirs(model_path)

    # Trains model on questions
    proscess_questions(questions_from_db, proscessed_data_path)
    remove_labels(proscessed_data_path + 'processed_data.csv', minimum_occurrence=5)
    produce_modell(proscessed_data_path + 'processed_data.csv',model_path)

    # Creates db entry for training event
    add_trained_entry(len(questions_from_db))

    # Changes the state of all the questions to trained_on
    change_state_of_question_to(questions_from_db,QuestionEnum.TRAINED_ON.value)

def train_with_csv(csv_data_path, proscessed_data_path, model_path):
    """ Takes the path to a CSV file with two values(value,answer) and trains a new model using this """
    # Getting the questions from the csv file
    questions_from_csv = get_train_from_csv(csv_data_path)

    # Making sure the path for the proscessed data exists
    if not os.path.exists(proscessed_data_path):
        os.makedirs(proscessed_data_path)
    # Making sure the path for the model exists
    if not os.path.exists(model_path):
        os.makedirs(model_path)

    # Trains model on questions
    proscess_questions(questions_from_csv,proscessed_data_path)
    remove_labels(proscessed_data_path + 'processed_data.csv', minimum_occurrence=5)
    produce_modell(proscessed_data_path + 'processed_data.csv',model_path)

    # Creates db entry for training event
    add_trained_entry(len(questions_from_csv))