from ..models import Question, Answer
from ..stateEnum import QuestionEnum
from .sorting_and_filtering import *
from datetime import datetime
from typing import List

import random
import string
from dateutil.relativedelta import relativedelta

def get_accuracy(questions: List[Question]) -> float:
    """ Gives the accuracy of a list of questions by dividing the number of questions where bot_answer_id == true_answer_id by the total number of questions.
    
    The fuction only counts questions where the state == 'validated'. """
    length = 0
    score = 0
    for question in questions:
        if is_validated(question) or is_trained_on(question):
            length=length+1
            if question.bot_answer_id == question.true_answer_id:
                score = score+1
    if length > 0:
        return score/length
    else:
        return 0

def confusion_matrix_to_dim(questions: List[Question], answer: Answer) -> List[List[int]]:
    """
    Gives a 2x2 matrix that representate the ConfusionMatrix
    for one answer in a sett of questions
    """
    confusionMatrix = [[0, 0], [0, 0]]
    for question in questions:
        if question.bot_answer_id == answer.id:
            # True Positiv
            if question.bot_answer_id == question.true_answer_id:
                confusionMatrix[0][0] += 1
            # False positive
            else:
                confusionMatrix[1][0] += 1
        # False Negative
        elif question.true_answer_id == answer.id:
            confusionMatrix[0][1] += 1
        #True Negative  
        else:
            confusionMatrix[1][1] += 1
    return confusionMatrix

def precision(confusionMatrix: List[List[int]]) -> float:
    """
    Give a Precision base on a confusingmatrix represent by a 2x2 matrix
    """
    #truePositive / (truePositive + falsePositive)
    if confusionMatrix[0][0] <= 0 or confusionMatrix[1][0] < 0:
        return 0
    else:
        return confusionMatrix[0][0] / (confusionMatrix[0][0] + confusionMatrix[1][0])

def recall(confusionMatrix: List[List[int]]) -> float:
    """
    Give a Recall base on a confusingmatrix represent by a 2x2 matrix
    """
    #truePositive / (truePositive + falseNegative)
    if confusionMatrix[0][0] <= 0 or confusionMatrix[0][1] < 0:
       return 0
    else:
        return confusionMatrix[0][0] / (confusionMatrix[0][0] + confusionMatrix[0][1])

def get_f1_score(confusionMatrix: List[List[int]]) -> float:
    """
    Give a F1-Score base on a confusingmatrix represent by a 2x2 matrix
    """
    pre = precision(confusionMatrix)
    re = recall(confusionMatrix)

    if (pre + re) <= 0:
        return 0
    else:
        return  2 * (pre * re) / (pre + re)
    

def get_num_of_questions(state: str) -> int:
    """ Gets the number of all questions with a given state. """
    return len(get_questions_by_state(state))

def get_the_increase_of_question(intervalMonth: int, currentDate: datetime = None) -> List[int] and List[str]:
    """Return a list of int that represent the number of incoming questions to that day, whit te given interval"""
    numberOfQuestionEachInteval = []
    dates = []
    allQuestions = get_all_questions()
    questionCount = 0
    if currentDate is None:
        currentDate = datetime.now()
    if len(allQuestions) != 0:
        startDate = min(question.timestamp for question in allQuestions)
        while startDate <= currentDate:
            startDate = startDate + relativedelta(months=intervalMonth)
            questionsUpToStartDate = [question for question in allQuestions if question.timestamp <= startDate]
            questionCount = len(questionsUpToStartDate)

            numberOfQuestionEachInteval.append(questionCount)
            dates.append(startDate.strftime("%b") + " " + str(startDate.year))

    return numberOfQuestionEachInteval, dates
    

#def get_number_of_years(end_date: datetime) -> int: 
 #   """Return the number startet years from the first askt question and to the given end_date"""
  #  questions = get_valid_questions_by_time_stat(datetime.min, datetime.now())  
   # min_year = end_date.year

    #if questions:
     #   min_date = min(question.timestamp for question in questions)  
      #  min_year = min_date.year
    #return end_date.year - min_year + 1


def get_number_of_years(end_date: datetime) -> int: 
    """Return the number startet years from the first askt question and to the given end_date"""
    questions = get_valid_questions_by_time_stat(datetime.min, datetime.now())  
    min_date = end_date

    if questions:
        min_date = min(question.timestamp for question in questions)

    if min_date > end_date:
        return 0
    
    if end_date.month < min_date.month or (end_date.month == min_date.month and end_date.day < min_date.day):
        return end_date.year - min_date.year
    
    #startet on a new year
    else:
        return end_date.year - min_date.year + 1 
    
def generate_colors(num_colors: int) -> List[str]:
    """Givs a list of x random colors, where x is defins as inputt: num_colors"""
    colors = set()
    while len(colors) < num_colors:
        color = '#' + ''.join(random.choices(string.hexdigits[:-6], k=6))
        colors.add(color)
    return list(colors)

# Sesion -------------------------------------------------------------------------------------------------------------------

def uppdate_count_sesion(session):
    """Uppdate all sesion that count number of instances"""
    session['countAll']=len(get_questions_by_state('all'))
    session['countValidate']=len(get_questions_by_state(QuestionEnum.VALIDATED.name))
    session['countNotValidate']=len(get_questions_by_state(QuestionEnum.NOT_VALIDATED.name))
    session['CountTrainOn']=len(get_questions_by_state(QuestionEnum.TRAINED_ON.name))