from typing import List
from ..models import Question
import pandas as pd
import nltk
nltk.download('punkt')
nltk.download('stopwords')
import string
import re

def clean_text(message):
		#Splitting sentence into words
        tokens = nltk.word_tokenize(message)
		#Lower case word
        tokens = [token.lower() for token in tokens]
		#punctuation
        punct_regex = re.compile('[{}]'.format(re.escape(string.punctuation)))
        tokens = [a for a,b in zip(tokens, [punct_regex.sub('', token) for token in tokens]) if b != '']
		#Removing stoppwords
        stopwords = nltk.corpus.stopwords.words('norwegian')
        tokens = [token for token in tokens if token not in stopwords]
		#Stemmer
        ss = nltk.stem.SnowballStemmer("norwegian")
        tokens = [ss.stem(word) for word in tokens]
        return [' '.join(token for token in tokens)]

def fix_labels(df):
        answer_dict = {
        #Feil skrevet kattegorier
        'buy_tickets': 'buy_tickets',
        'program_content': 'program_content',
        'gavekort': 'gavekort',
        'festspillgavekort': 'gavekort',
        'get_dress_code': 'get_dress_code',
        'find_festival_dates': 'find_festival_dates',
        'find_prices': 'find_prices',
        'get_refund': 'get_refund',
        'unknown ': 'unknown',
        'true': 'true',
        'unknown': 'unknown',
        'get_bicycle': 'get_bicycle',
        'print_tickets': 'print_tickets',
        'volunteer': 'volunteer',
        'find_festival dates': 'find_festival_dates',
        'ticketing_request': 'ticketing_request',
        'program_content ': 'program_content',
        'FiB Fordel-info ': 'find_rebates',
        'unkown': 'unknown',
        # np.nan: np.nan,
        'find_rebates': 'find_rebates',
        'get_rebates': 'find_rebates',
        'festspillkort': 'festspillkort',
        'Unknown': 'unknown',
        'true ': 'true',
        #Slå saman kattegorier
        'ticketing_request': 'buy_tickets',
        'find_rebates': 'find_prices',
        #Nytt navn på kattegorier:
        'get_bicycle':'goodbye'
        }

        #Fikse opp i kategoriane
        df["answer"].replace(answer_dict).nunique()
        df["answer"].replace(answer_dict, inplace=True)

        return df

def proscess_questions(questions:List[Question],save_path:str):
        """ Makes sure data is ready for model to train on 
        
        :param questions: List of questions you want to train your model on. 
        :param save_path: Path to where you want to save your proscessed data. """
        # Making dataframe with questions
        df = pd.DataFrame(columns=['message', 'label'])
        for question in questions:
            message = question.message
            label = question.true_answer_id
            new_row = pd.Series({'message': message, 'label': label})
            df = pd.concat([df,new_row.to_frame().T], ignore_index=True)
       
        # Clean the text in the "message" column of the DataFrame
        cleaned_text = [clean_text(message) for message in df["message"]]
        # Join the cleaned text into single strings using spaces as separators
        joined_text = [' '.join(token for token in tokenized_message) for tokenized_message in cleaned_text]
        # Create a new DataFrame with the cleaned text and the original labels
        df_new = pd.DataFrame({'message': joined_text, 'label': df["label"]})
        # Save the new DataFrame to a CSV file
        df_new.to_csv(save_path + 'processed_data.csv', index=False)

def remove_labels(csv_file,minimum_occurrence=5):
    df = pd.read_csv(csv_file)
    label_counts = df['label'].value_counts()
    valid_labels = []

    # Appends label to valid_labels if it occures in more questions than minimum_occurence
    for label, count in label_counts.items():
        if count >= minimum_occurrence:
            valid_labels.append(label)

    # Makes a new empty df with two columns
    new_df = pd.DataFrame(columns=['message','label'])

    # Adds every question with a valid label to the new df
    for valid_lable in valid_labels:
          #new_df = new_df.append(df[df['label'] == valid_lable])
          new_df = pd.concat([new_df,df[df['label'] == valid_lable]])

    #overwrites the content of the original csv-file with the info from the new df
    new_df.to_csv(csv_file, index=False)
