import pandas as pd
import pickle

from sklearn.feature_extraction.text import TfidfVectorizer

#Import av modeller:
from sklearn import linear_model
from sklearn.linear_model import SGDClassifier
from sklearn import svm
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.neural_network import MLPClassifier

#Blender modeller
from sklearn.ensemble import StackingClassifier
from sklearn.linear_model import LogisticRegression


def setningSamensloer(kategori, df):
    dokument = ""
    for ele in df[df.label == kategori]["message"]:
        dokument += " " + str(ele)
    return dokument

def produce_modell(file, save_path):
    df = pd.read_csv(file)
    labels = df.groupby("label").count().index.to_list()

    print("Create the tf-idf model")
    kategorierSamanslott = pd.DataFrame({'dokument': [setningSamensloer(x, df) for x in labels], 
                                        'kategori': labels
                                        })

    tfidf_vectorizer = TfidfVectorizer()
    tfidf_vectorizer.fit(kategorierSamanslott["dokument"])
    tfidf_Data = tfidf_vectorizer.transform(df["message"].values.astype('U'))
    
    #Modeller som blir brukt
    estimators = [
        ('clf', linear_model.LogisticRegression(solver='liblinear', max_iter=100)),
        ('sgd', SGDClassifier(loss='hinge', penalty="l2", max_iter=60, random_state=4)),
        ('svc', svm.SVC(kernel='linear', C = 1.0, probability=True)), 
        ('ranforC', RandomForestClassifier(max_depth=128, random_state=0)),
        ('gbc', GradientBoostingClassifier(n_estimators=100, learning_rate=1.0, max_depth=6, random_state=0)),
        ('ctc', DecisionTreeClassifier(random_state=0)),
        ('mlp', MLPClassifier(solver='lbfgs', random_state=42, max_iter=100))
    ]

    #Blender modellen som er den endelege modellen
    print("Starting model training. this can take some time...")
    ctf = StackingClassifier(estimators=estimators, final_estimator=LogisticRegression(max_iter=1000, n_jobs=-1))
    ctf.fit(tfidf_Data, df["label"])

    print("Saving the model")
    #ML-modell
    pickle.dump(ctf, open(save_path + "blender_model_v2.sav", 'wb'))
    #tf_idf2 modell
    pickle.dump(tfidf_vectorizer, open(save_path + "tf-idf2_model_v2.pickle", 'wb'))

#produce_modell("data/processed/processed_data.csv", "data/processed/")