from flask import Blueprint, render_template, request, flash, redirect, url_for, session
from flask_login import login_required, current_user
from datetime import datetime, timedelta
from .stateEnum import QuestionEnum
import requests
from . import db
from collections import defaultdict

from .helpfunctions.dymmyData import *
from .helpfunctions.database import *
from .helpfunctions.processDataFile import *
from .helpfunctions.sorting_and_filtering import *
from .helpfunctions.statistics import *
from .helpfunctions.authorization import *

from logging import getLogger
logger = getLogger('FiB-CA_log')

views = Blueprint('views',__name__)


""" Route for home page """
@views.route('/',methods=['GET','POST'])
@custom_login_required()
def home():
    if not session.get('state'):
        session["state"]=QuestionEnum.NOT_VALIDATED.name
    if not session.get('sort_by'):
        session['sort_by']='timestamp_last_first'
    if not session.get('max_questions'):
        session['max_questions']="20"
    if not session.get('show_question_by_true_answer'):
        session['show_question_by_true_answer']="all"

    # Number of question by state
    if request.method == 'GET':
        uppdate_count_sesion(session)

    if request.method == 'POST':

        form = request.form.get('form')

# Menues for what questions to show --------------------------------------------------------------------------------------

        if form == 'choose_state':
            show_value = request.form.get('show_value')
            session["state"] = show_value

        elif form == 'sort_by':
            show_value = request.form.get('show_value')
            session["sort_by"] = show_value

        elif form == 'number_of_questions':
            show_value = request.form.get('show_value')
            session['max_questions'] = show_value
        # show question by tru_answer dropdown menu
        elif form == 'show_question_by_true_answer':
            show_value = request.form.get('show_value')
            session['show_question_by_true_answer'] = show_value

        elif form == 'show_date':
            action = request.form.get('action') 
            start = request.form.get('start')
            end = request.form.get('end')

            if action == 'clear_start': #clear start date
                if 'show_from' in session:
                    session.pop('show_from')
            elif action == 'clear_end': #clear end date
                if 'show_to' in session:
                    session.pop('show_to')
            else:
                if start and end:
                    session['show_from'] = datetime.strptime(start, '%Y-%m-%d')
                    session['show_to'] = datetime.strptime(end, '%Y-%m-%d')
                elif start and not end:
                    session['show_from'] = datetime.strptime(start, '%Y-%m-%d')
                    if 'show_to' in session:
                        session.pop('show_to')
                elif not start and end:
                    if 'show_from' in session:
                        session.pop('show_from')
                    session['show_to'] = datetime.strptime(end, '%Y-%m-%d')
                else:
                    if 'show_from' in session:
                        session.pop('show_from')
                    if 'show_to' in session:
                        session.pop('show_to')

# Delete and save questions -----------------------------------------------------------------------------------------------

        elif form == 'delete_question':
            queston_id = request.form.get('queston_id')
            question = get_question(int(queston_id))
            if question:
                delete_question(question)
                uppdate_count_sesion(session)
            else:
                print ("delete question not found")

        elif form == 'save_questions':
            questions = get_questions_by_state(session.get('state'))
            questions = filter_questions_with_timestamp(questions, session.get('show_from'), session.get('show_to'))
            questions = sort_questions_by(questions,session.get('sort_by'))
            questions = filter_by_answer_lable(questions, session.get('show_question_by_true_answer'))
            questions = questions[:int(session.get('max_questions'))]

            validated_questions = []
            changed_questions = []
            for question in questions:
                true_answer_id = int(request.form.get('true_answer_id_'+str(question.id)))
                box = request.form.get('checked_'+str(question.id))
                
                # Checkes if the checkbox for each
                checked = False
                if box : checked = True
               
                if true_answer_id:
                    # Checks if any values are changed
                    if is_validated(question) != checked and not is_trained_on(question):
                        validated_questions.append(question.id)
                    if question.true_answer_id != true_answer_id:
                        changed_questions.append(question.id)
                    # Saves changes
                    if (checked and not is_trained_on(question)) or (is_trained_on(question) and question.true_answer_id != true_answer_id):
                        question.true_answer_id = true_answer_id
                        question.state = QuestionEnum.VALIDATED.value
                        db.session.commit()
                    elif not checked:
                        question.true_answer_id = true_answer_id
                        question.state = QuestionEnum.NOT_VALIDATED.value
                        db.session.commit()
            # Logs changes
            if len(validated_questions)>0:
                logger.info('User id:'+str(current_user.id)+' validated question id\'s:'+str(validated_questions)+'.')
            if len(changed_questions)>0:
                logger.info('User id:'+str(current_user.id)+' changed true_answer_id in question id\'s:'+str(changed_questions)+'.')
            uppdate_count_sesion(session)
            flash('Spørsmål validert!',category='success')
            
    answers = sorted(get_all_answers(), key=lambda x: x.description)
    questions = get_questions_by_state(session.get('state'))
    questions = filter_questions_with_timestamp(questions, session.get('show_from'), session.get('show_to'))
    questions = filter_by_answer_lable(questions, session.get('show_question_by_true_answer'))
    questions = sort_questions_by(questions,session.get('sort_by'))
    countSelectetQuestions = len(questions)

    questions = questions[:int(session.get('max_questions'))]
    
    return render_template("home.html",user=current_user,
                           questions=questions,
                           answers=answers,
                           getAnswer=get_answer, 
                           isValidated=is_validated, 
                           isTrainedOn=is_trained_on, 
                           QuestionEnum=QuestionEnum,
                           countSelectetQuestions=countSelectetQuestions,
                           isActive=is_active)

""" Route for answer categor page """
@views.route('/kategori',methods=['GET','POST'])
@custom_login_required()
def cat():
    if request.method == 'POST':

        form = request.form.get('form')

        if form == 'save_answers':
            answers = get_all_answers() # can use getStandardAnswers() and getSeasonAnswers() if you dont want to save both
            for answer in answers:
                description = request.form.get('description_'+str(answer.id))
                answer_content = request.form.get('answer_'+str(answer.id))
                if description:
                    if is_safe(answer_content):
                        answer.description = description
                        answer.answer = answer_content
                        db.session.commit()
                        
                    else: 
                        flash('Ulovelig tegnsetting brukt i "'+ answer.description +'".',category='error')
            flash('Endringer er lagret!',category='success')
        
        if form == 'toggle_answer_state':
            answer_id = request.form.get('answer_id')
            answer = get_answer(answer_id)
            if answer:
                print('toggling: '+answer.description)
                toggle_answer_state(answer)

        if form == 'add_season':

            answer = request.form.get('answer')
            description= request.form.get('description')
            label = to_lable(description)
            if len(label) > 0 and is_safe(label) and is_safe(answer):
                state = AnswerEnum.SEASON.value
                print("lable= " + label)
                new_answer = Answer(answer=answer,lable=label, description=description,state=state) 
                db.session.add(new_answer)
                db.session.commit()
                flash('Sesongsvar lagt til! ('+ new_answer.description +')',category='success')
            flash('Nytt sesongsvar er ulovligt".',category='error')


        if form == 'add_keyword':
            word = request.form.get('word')
            answer_id = request.form.get('answer_id')

            answer = Answer.query.get(answer_id)
            if answer and word:
                add_keyword(answer,word)
                flash('Nøkkelord "' + word +'" lagt til i '+ answer.description ,category='success')
        
        if form == 'delete_keyword':
            keyword_id = request.form.get('keyword_id')
            keyword = Keyword.query.get(keyword_id)
            answer_id = request.form.get('answer_id')
            answer = Answer.query.get(answer_id)
            if keyword:
                remove_keyword(keyword)
                flash('Nøkkelord "'+ keyword.word + '" fjernet fra ' + answer.description, category='success')


    standard_answers = sorted(get_standard_answers(), key=lambda x: (not is_active(x), x.description))
    season_answers = sorted(get_season_answers(), key=lambda x: (not is_active(x), x.description))
    return render_template("cat.html",user=current_user, 
                           standard_answers=standard_answers, 
                           season_answers=season_answers, 
                           isActive=is_active, 
                           getKeywordsByAnswer=get_keywords_by_answer)


""" Route for train page """
@views.route('/tren',methods=['GET','POST'])
@custom_login_required()
def train():
    if request.method == 'POST':
        
        form = request.form.get('form')
            
        if form == 'train':

            proscessed_data_path = 'fib_chatbot/src/data/proscecced/'
            model_path = 'fib_chatbot/src/data/model/'
            csv_file = os.path.join(os.path.dirname(__file__), "datadump/questions.csv")

            # You can train a new model with the following functions: trainWithDb, trainWithCsv, trainWithCsvAndDb
            #trainWithDb(proscessed_data_path,model_path)
            train_with_db(proscessed_data_path,model_path)
            #train_with_csv_and_db(csv_file, proscessed_data_path, model_path)
            flash('Chatboten er retrent!' ,category='success')

            get_last_trained().datapoints
            # Logs event
            logger.info('User id:'+str(current_user.id)+' Trained model on '+str(get_last_trained().datapoints)+' questions.')

    last_datetime = get_last_trained().timestamp
    last_trained = "den "+last_datetime.strftime("%d/%m/%Y")+" klokken "+last_datetime.strftime("%H:%M")
    trained_on = get_last_trained().datapoints
    train_on = get_num_of_questions(QuestionEnum.VALIDATED.name)+get_num_of_questions(QuestionEnum.TRAINED_ON.name)
    return render_template("train.html",user=current_user,
                           train_on=train_on,
                           last_trained=last_trained,
                           trained_on=trained_on,
                           QuestionEnum=QuestionEnum)



""" Route for statistics page """
@views.route('/stats', methods=['GET', 'POST'])
@custom_login_required()
def stats():

    #Default time period
    if not session.get('show_stats_from') or not session.get('show_stats_to'):
        # Set default values for start and end dates
        end_date = datetime.now()
        start_date = end_date - timedelta(days=365)
        session['show_stats_from'] = start_date
        session['show_stats_to'] = end_date 
    
    if not session.get('page'):
        session['page'] = 1

    if request.method == 'POST':
        form = request.form.get('form_type')

        if form == 'show_stats':
            #Uppdate time period
            # Get start and end dates from form submission
            start_date = datetime.strptime(request.form['start'], '%Y-%m-%d')
            end_date = datetime.strptime(request.form['end'], '%Y-%m-%d')

            # Store selected dates in session for future requests
            session['show_stats_from'] = start_date
            session['show_stats_to'] = end_date

            #Uppdate number of shown year  
            show_value = request.form.get('show_value')
            if (show_value == 'all'):
                session['numberOfYears'] = get_number_of_years(end_date)
            else:
                session['numberOfYears'] = show_value

            #Save the active page in session
            
            page1 = request.form.get('page1')
            page2 = request.form.get('page2')
            page3 = request.form.get('page3')
            if (page1 == '1'):
                session['page'] = 1
            elif (page2 == '1'):
                session['page'] = 2
            elif (page3 == '1'):
                session['page'] = 3

    #get start an end date
    start_date = session.get('show_stats_from')
    end_date = session.get('show_stats_to')

    #in no sesion Uppdate number og shown year
    if not session.get('numberOfYears'):
        session['numberOfYears'] = get_number_of_years(end_date)

    #Initial Tabel
    numberOfYears = int(session.get('numberOfYears'))
    accuracyPerSeason = [0]*numberOfYears
    questionsPerYear = [0]*numberOfYears
    numberOfQuestionsPerYear = [0]*numberOfYears
    question_count_by_answer = [0]*numberOfYears

    #Uppdate table
    for i in range(numberOfYears):
        questionsPerYear[i] = get_valid_questions_by_time_stat(start_date, end_date)
        numberOfQuestionsPerYear[i] = len(questionsPerYear[i])
        # Get the question count by answer for the selected time period
        question_count_by_answer[i] = get_valid_question_count_by_true_answer(start_date, end_date)
        end_date -= timedelta(days=365)
        start_date -= timedelta(days=365)
    
    #Show all categories when comparing years
    category_question_counts = defaultdict(lambda: defaultdict(int))

    for i in range(numberOfYears):
        for answer in question_count_by_answer[i]:
            category_question_counts[answer['description']][i] = answer['count']

    #Sort alphabetical
    #all_categories = sorted(category_question_counts.keys())

    #Sort by question count
    all_categories = sorted(category_question_counts.keys(), key=lambda x: category_question_counts[x]['current_year'], reverse=True)

    #Get the accuracy for the folowing years
    for i in range(numberOfYears):
        accuracyPerSeason[i] = round(get_accuracy(questionsPerYear[i])*100, 2)

    #Get f1score
    validQuestions = get_questions_by_state(QuestionEnum.VALIDATED.name)
    validQuestions += get_questions_by_state(QuestionEnum.TRAINED_ON.name)

    answers = get_active_answers()

    #Answer whit qyestions
    answers_whit_questions_description = [answer for answer in all_categories]
    

    #Get the answer element in the right order
    answers_whit_questions = []
    for answer_description in all_categories:
        for answer in answers:
            if answer.description == answer_description:
                answers_whit_questions.append(answer)


    #add answer without questions:
    for answer in answers:
        if not all_categories.__contains__(answer.description):
            all_categories.append(answer.description)

    f1Score = [round(get_f1_score(confusion_matrix_to_dim(validQuestions, answer))*100, 2) for answer in answers_whit_questions]

    #Sum all years base on each answer
    totalQuestionPerAnswer = [0*len(answers)]
    totalQuestionPerAnswer = [sum([category_question_counts[category][i]for i in range(numberOfYears)]) for category in all_categories]
    
    #For every answer it is one f1-score for every year. [answer][year]
    f1ScoreByYear = [[] for _ in range(len(answers_whit_questions))]
    for i in range(len(answers_whit_questions)):
        for j in range(numberOfYears):
            f1ScoreByYear[i].append(round(get_f1_score(confusion_matrix_to_dim(questionsPerYear[j], answers_whit_questions[i]))*100, 2))

    #Uppdate colors if nessesary:
    colors = session.get('colors')
    if not colors:
        colors = generate_colors(len(answers))
        session['colors'] = colors
    elif len(colors) != len(answers):
        colors = generate_colors(len(answers))
        session['colors'] = colors

    numberOfQuestionEachInteval, dates = get_the_increase_of_question(1)

    totalAccuracy = round(get_accuracy(validQuestions)*100, 2)
    totalQuestion = sum(numberOfQuestionsPerYear, 0)

    end_date=session.get('show_stats_to')
    years = [end_date.year - i for i in range(numberOfYears)]

    # Render the template with the selected dates and statistics
    return render_template("stats.html", user=current_user, 
                            start_date=start_date, 
                            end_date=end_date, 
                            datetime=datetime, 
                            question_count_by_answer=question_count_by_answer,
                            all_categories=all_categories,
                            category_question_counts=category_question_counts,
                            accuracyPerSeason=accuracyPerSeason,
                            totalAccuracy = totalAccuracy,
                            numberOfQuestionsPerYear = numberOfQuestionsPerYear,
                            totalQuestion = totalQuestion,
                            numberOfYears=numberOfYears,
                            answers_whit_questions_description = answers_whit_questions_description,
                            numberOfAnswers=len(answers),
                            f1Score=f1Score,
                            f1ScoreByYear=f1ScoreByYear,
                            colors = colors,
                            totalQuestionPerAnswer=totalQuestionPerAnswer,
                            years=years,
                            numberOfQuestionEachInteval=numberOfQuestionEachInteval,
                            dates=dates
                            )

""" Route for bot page """
@views.route('/bot',methods=['GET','POST'])
@custom_login_required()
def bot():

    botTilstand = "Test"
    botAnswer = ""
    if request.method == 'POST':

        form = request.form.get('form')
        if form == 'testBot':
            statusChatbotJson = requests.get(url = "http://127.0.0.1:5000/fib_chatbot/api/status")
            if statusChatbotJson == None:
                botTilstand = "Ingen respons"
            else:
                botTilstand = statusChatbotJson.text
        if form == 'askQuestion':
            data = [{"message": request.form.get("question")}]
            headers = {"Content-Type": "application/json"}
            url = "http://127.0.0.1:5000/fib_chatbot/api/predict"
            botRespons = requests.post(url = url, headers = headers, json = data)
            #botAnswer=.json()
            if botRespons.status_code == 200:
                j = botRespons.json()
                answer_id = int(j[0]['predicted'])
                botAnswer = get_answer(answer_id)
            else:
                flash('chatbot error code: ' + str(botRespons.status_code) ,category='error')

        elif form == 'generate_data':
            show_value = request.form.get('show_value')
            if show_value == 'createAnswers':
                gennerate_answer()
            elif show_value == 'createQuestions':
                generate_questions()
            elif show_value == 'createDatadump':
                fill_database_with_json()
    return render_template("bot.html",user=current_user, 
                           botTilstand=botTilstand, 
                           botAnswer=botAnswer)
